//===================================================================
// DamageAmplifierlifier
//===================================================================
class DamageAmplifier expands UDamage;

var() name WearedOffEvent;
var() name DroppedEvent;
var() name ReActivateEvent;

var bool bDroppedNWaiting;

var name WOEvent[32];
var name DEvent[32];
var name RAEvent[32];

replication
{
    reliable if( Role==ROLE_Authority)
	bDroppedNWaiting;
}

simulated function Destroyed()
{
local Actor A;
local byte bj;

	if (WearedOffEvent != '' && (Charge <= 0 || bDroppedNWaiting))
	{
	ForEach AllActors( class 'Actor', A, WearedOffEvent)
                		A.Trigger( Self, Instigator );

		For (bj=0; bj<32; bj++)
		{
			if (WOEvent[bj] != '')
			{
				ForEach AllActors( class 'Actor', A, WOEvent[bj])
                			A.Trigger( Self, Instigator );
			}
		}
	}

	if (DroppedEvent != '' && Charge > 0 && Pawn(Owner)!=None)
	{
	ForEach AllActors( class 'Actor', A, DroppedEvent)
                		A.Trigger( Self, Instigator );

		For (bj=0; bj<32; bj++)
		{
			if (DEvent[bj] != '')
			{
				ForEach AllActors( class 'Actor', A, DEvent[bj])
                			A.Trigger( Self, Instigator );
			}
		}
	}


	if (Charge > 0 && !bDroppedNWaiting)
		DropACopy();

	if (Pawn(Owner) == None && !bHidden)
		Level.Game.PlaySpawnEffect(Self);

	Super.Destroyed();
}

function DropACopy()
{
local DamageAmplifier UDAM;
local vector PossibleLoc, HitLocation, HitNormal;
local actor AT;
local byte bj;

	PossibleLoc = Pawn(Owner).Location + (Rand(128) - 64)*vect(1,0,0) + (Rand(128) - 64)*vect(0,1,0);
	AT = Trace (HitLocation, HitNormal, PossibleLoc, Pawn(Owner).Location, False);

	if (AT == None)
		UDAM = Spawn(Class'DamageAmplifier', None,, PossibleLoc, Rand(32768) * rot(0,1,0));
	else
		UDAM = Spawn(Class'DamageAmplifier', None,, Pawn(Owner).Location, Rand(32768) * rot(0,1,0));

	UDAM.bDroppedNWaiting = True;
	UDAM.Charge = Charge;
	UDAM.Event = Event;
	UDAM.WearedOffEvent = WearedOffEvent;
	UDAM.SetPhysics(PHYS_Falling);
	UDAM.bRotatingPickup = False;
	UDAM.RotationRate = rot(0,0,0);
	UDAM.Tag = Tag;
	UDAM.bFixedRotationDir = False;
	UDAM.ReActivateEvent = ReActivateEvent;
	UDAM.DroppedEvent = DroppedEvent;
	UDAM.LifeSpan = 15.0; // Lifespan of the new dropped copy before it gets destroyed if nobody picks it up

	For (bj=0; bj<32; bj++)
	{
		UDAM.WOEvent[bj] = WOEvent[bj];
		UDAM.DEvent[bj] = DEvent[bj];
		UDAM.RAEvent[bj] = RAEvent[bj];
	}
}

simulated function Tick(float DeltaTime)
{
	if ((Region.Zone.bDestructive || Region.Zone.bKillZone || Region.Zone.DamagePerSec > 0) && Pawn(Owner) == None)
	{
		Charge = 0;
		Destroy();
	}
		
}

function inventory SpawnCopy( pawn Other )
{
    local inventory Copy;

    if( Level.Game.ShouldRespawn(self) && !bDroppedNWaiting)
    {
        Copy = spawn(Class,Other,,,rot(0,0,0));
        Copy.Tag           = Tag;
        Copy.Event         = Event;
        GotoState('Sleeping');
    }
    else
        Copy = self;

    Copy.RespawnTime = 0.0;
    Copy.bHeldItem = true;
	DamageAmplifier(Copy).WearedOffEvent = WearedOffEvent;
	DamageAmplifier(Copy).DroppedEvent = DroppedEvent;
	DamageAmplifier(Copy).ReActivateEvent = ReActivateEvent;
    Copy.Charge = Charge;
    Copy.GiveTo( Other );
    return Copy;
}


function bool HandlePickupQuery( inventory Item )
{
local byte bj, bi;
local bool bHaveThisEventReg;

    if (item.class == class) 
    {
        if ( Charge < 999 )  
	{
                Charge += Item.Charge;
		if (Charge > 999)
			Charge = 999;

		//Events management
		///////////////////////////////////////////////////////////

		if (WearedOffEvent == '')
			WearedOffEvent = DamageAmplifier(Item).WearedOffEvent;
		if (DroppedEvent == '')
			DroppedEvent = DamageAmplifier(Item).DroppedEvent;

		if (WearedOffEvent != DamageAmplifier(Item).WearedOffEvent)
		{
			bHaveThisEventReg = False;
			For (bj=0; bj<32; bj++)
			{
				if (WOEvent[bj] == DamageAmplifier(Item).WearedOffEvent)
				{
					bHaveThisEventReg = True;
					break;
				}
			}

			if (!bHaveThisEventReg)
			{
				For (bj=0; bj<32; bj++)
				{
					if (WOEvent[bj] == '')
					{
						WOEvent[bj] = DamageAmplifier(Item).WearedOffEvent;
						break;
					}
				}
			}
		}

		if (DroppedEvent != DamageAmplifier(Item).DroppedEvent)
		{
			bHaveThisEventReg = False;
			For (bj=0; bj<32; bj++)
			{
				if (DEvent[bj] == DamageAmplifier(Item).DroppedEvent)
				{
					bHaveThisEventReg = True;
					break;
				}
			}

			if (!bHaveThisEventReg)
			{
				For (bj=0; bj<32; bj++)
				{
					if (DEvent[bj] == '')
					{
						DEvent[bj] = DamageAmplifier(Item).DroppedEvent;
						break;
					}
				}
			}
		}

		if (ReActivateEvent != DamageAmplifier(Item).ReActivateEvent)
		{
			bHaveThisEventReg = False;
			For (bj=0; bj<32; bj++)
			{
				if (RAEvent[bj] == DamageAmplifier(Item).ReActivateEvent)
				{
					bHaveThisEventReg = True;
					break;
				}
			}

			if (!bHaveThisEventReg)
			{
				For (bj=0; bj<32; bj++)
				{
					if (RAEvent[bj] == '')
					{
						RAEvent[bj] = DamageAmplifier(Item).ReActivateEvent;
						break;
					}
				}
			}
		}

		For (bj=0; bj<32; bj++)
		{
			if (DamageAmplifier(Item).WOEvent[bj] != '')
			{
				bHaveThisEventReg = False;
				For (bi=0; bi<32; bi++)
				{
					if (DamageAmplifier(Item).WOEvent[bj] == WOEvent[bi])
					{
						bHaveThisEventReg = True;
						break;
					}
				}

				
				if (!bHaveThisEventReg)
				{
					For (bi=0; bi<32; bi++)
					{
						if (WOEvent[bi] == '')
						{
							WOEvent[bi] = DamageAmplifier(Item).WOEvent[bj];
							break;
						}
					}
				}
				
			}
			else
				break;
		}


		For (bj=0; bj<32; bj++)
		{
			if (DamageAmplifier(Item).DEvent[bj] != '')
			{
				bHaveThisEventReg = False;
				For (bi=0; bi<32; bi++)
				{
					if (DamageAmplifier(Item).DEvent[bj] == DEvent[bi])
					{
						bHaveThisEventReg = True;
						break;
					}
				}

				
				if (!bHaveThisEventReg)
				{
					For (bi=0; bi<32; bi++)
					{
						if (DEvent[bi] == '')
						{
							DEvent[bi] = DamageAmplifier(Item).DEvent[bj];
							break;
						}
					}
				}
				
			}
			else
				break;
		}

		For (bj=0; bj<32; bj++)
		{
			if (DamageAmplifier(Item).RAEvent[bj] != '')
			{
				bHaveThisEventReg = False;
				For (bi=0; bi<32; bi++)
				{
					if (DamageAmplifier(Item).RAEvent[bj] == RAEvent[bi])
					{
						bHaveThisEventReg = True;
						break;
					}
				}

				
				if (!bHaveThisEventReg)
				{
					For (bi=0; bi<32; bi++)
					{
						if (RAEvent[bi] == '')
						{
							RAEvent[bi] = DamageAmplifier(Item).RAEvent[bj];
							break;
						}
					}
				}
				
			}
			else
				break;
		}

			

		if (Event == '')
			Event = DamageAmplifier(Item).Event;
		////////////////////////////////////////////////////////////////////////////////////
	}
            if (Level.Game.LocalLog != None)
                Level.Game.LocalLog.LogPickup(Item, Pawn(Owner));
            if (Level.Game.WorldLog != None)
                Level.Game.WorldLog.LogPickup(Item, Pawn(Owner));
            if ( Item.PickupMessageClass == None )
                Pawn(Owner).ClientMessage(item.PickupMessage, 'Pickup');
            else
                Pawn(Owner).ReceiveLocalizedMessage( item.PickupMessageClass, 0, None, None, item.Class );
            Item.PlaySound (item.PickupSound,,2.0);
            Item.SetReSpawn();

        return true;        
    }

    if ( Inventory == None )
        return false;

    return Inventory.HandlePickupQuery(Item);
}


simulated function FireEffect()
{
	if ( (TimerRate - TimerCounter < 5) && (Level.NetMode != NM_Client) )
		Pawn(Owner).Weapon.PlayOwnedSound(EndFireSound, SLOT_Interact, 8);
	else 
		Pawn(Owner).Weapon.PlayOwnedSound(ExtraFireSound, SLOT_Interact, 8);
}


function SetOwnerLighting()
{
	if ( Owner.bIsPawn && Pawn(Owner).bIsPlayer
		&& (Pawn(Owner).PlayerReplicationInfo.HasFlag != None) ) 
		return;
	Owner.AmbientGlow = 254; 
	Owner.LightEffect=LE_NonIncidence;
	Owner.LightBrightness=255;
	Owner.LightHue=210;
	Owner.LightRadius=10;
	Owner.LightSaturation=0;
	Owner.LightType=LT_Steady;
}

function SetUDamageWeapon()
{
	if ( !bActive )
		return;

	SetOwnerLighting();

	// Make old weapon normal again.
	if ( UDamageWeapon != None )
	{
		UDamageWeapon.SetDefaultDisplayProperties();
		if ( UDamageWeapon.IsA('TournamentWeapon') )
			TournamentWeapon(UDamageWeapon).Affector = None;
	}
		
	UDamageWeapon = Pawn(Owner).Weapon;
	// Make new weapon cool.
	if ( UDamageWeapon != None )
	{
		if ( UDamageWeapon.IsA('TournamentWeapon') )
			TournamentWeapon(UDamageWeapon).Affector = self;
		if ( Level.bHighDetailMode )
			UDamageWeapon.SetDisplayProperties(ERenderStyle.STY_Translucent, 
									 FireTexture'UnrealShare.Belt_fx.UDamageFX',
									 true,
									 true);
		else
			UDamageWeapon.SetDisplayProperties(ERenderStyle.STY_Normal, 
							 FireTexture'UnrealShare.Belt_fx.UDamageFX',
							 true,
							 true);
	}
}


state Activated
{

	function Timer()
	{

		Charge--;

		if (Charge <= FinalCount && Charge>0)
		{
			SetTimer(0.7, False);
			Owner.PlaySound(DeActivateSound,, 8);
		}
		else if (Charge <= 0)
		{
			Owner.PlaySound(DeActivateSound,, 8);
			UsedUp();
		}
		else
			SetTimer(1.0, False);
	}

	function BeginState()
	{
	local Actor A;
	local byte bj;
		
		bActive = true;
        	FinalCount = Min(FinalCount, Charge - 1);
		SetTimer(1.0,false);
        	Owner.PlaySound(ActivateSound); 
        	SetOwnerLighting();
        	Pawn(Owner).DamageScaling = 3.0;
        	SetUDamageWeapon();
		LifeSpan = 0.0;
		AmbientSound = None;

		if (ReActivateEvent != '' && bDroppedNWaiting)
		{
			ForEach AllActors( class 'Actor', A, ReActivateEvent)
                		A.Trigger( Self, Instigator );

		For (bj=0; bj<32; bj++)
		{
			if (RAEvent[bj] != '')
			{
				ForEach AllActors( class 'Actor', A, RAEvent[bj])
                			A.Trigger( Self, Instigator );
			}
		}

		}

		bDroppedNWaiting = False;
	}
}


State Sleeping
{
    ignores Touch;

    function BeginState()
    {
        BecomePickup();
        bHidden = true;
    }
    function EndState()
    {
        local int i;

        bSleepTouch = false;
        for ( i=0; i<4; i++ )
            if ( (Touching[i] != None) && Touching[i].IsA('Pawn') )
                bSleepTouch = true;
    }           
Begin:
    Sleep( ReSpawnTime );
    Sleep( PlaySpawnEffect(self) );
    GoToState( 'Pickup' );
}


function float PlaySpawnEffect(inventory Inv)
{
    spawn(class 'EnhancedReSpawn',Inv,, Inv.Location );
    return 0.3;
}

defaultproperties
{
    PickupMessage="-] Amplifier [-"
    Charge=30
}
